"""Build a geopackage from gSSURGO source gdb data."""

import gdal


def build_gpkg(in_gdb, out_gpkg):
    """Build a geopackage from gSSURGO source gdb data."""
    src_ds = gdal.OpenEx(in_gdb)
    ds = gdal.VectorTranslate(out_gpkg, srcDS = src_ds, format = "GPKG")
    del ds



"""Pull gSSURGO data based on mukeys."""

# https://gdal.org/python/
# https://gis.stackexchange.com/a/200477/32531

import os
import sys
import sqlite3
import gdal
import pandas as pd
import numpy as np
from pyproj import Proj, transform
from .aoi import state_by_bbox


def query_gpkg(src_tif, gpkg_path, sql_query, out_raster):
    r"""Pull gSSURGO data based on mukeys.

    Examples
    --------
    gssurgo.query_gpkg(src_tif = "tests/aoi.tif", gpkg_path = "gpkgs", \
                       sql_query = 'SELECT mukey, nonirryield_r \
                                    FROM mucropyld \
                                    WHERE (cropname = "Corn")', \
                                    out_raster = "tests/nonirryield_r.tif")

    """
    ds = gdal.Open(src_tif)

    gtransform = ds.GetGeoTransform()
    pixelWidth = gtransform[1]
    pixelHeight = gtransform[5]
    nrow = ds.RasterYSize
    ncol = ds.RasterXSize

    xmin = gtransform[0]
    ymax = gtransform[3]
    xmax = xmin + ncol * pixelWidth
    ymin = ymax - nrow * pixelHeight

    # https://gis.stackexchange.com/a/78944/32531
    outProj = Proj(init='epsg:4326')
    inProj = Proj('+proj=aea +lat_1=29.5 +lat_2=45.5 +lat_0=23 +lon_0=-96 \
                   +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m \
                   +no_defs')
    xmin, ymin = transform(inProj, outProj, xmin, ymin)
    xmax, ymax = transform(inProj, outProj, xmax, ymax)

    raw_values = ds.ReadAsArray()
    pixel_values = raw_values.flatten()
    pixel_values = pd.DataFrame(pixel_values, columns = ['mukey'])
    pixel_values.mukey = pixel_values.mukey.astype(int)

    # print(table.mukey.describe())
    # print(table[table.mukey.isin([186365, 1455241])])

    # find src gpkgs
    src_gpkg = state_by_bbox(fpath = gpkg_path, ext = "gpkg", xmin = xmin,
                             xmax = xmax, ymin = ymin, ymax = ymax)

    # read data and join to raster index
    if(len(src_gpkg) == 1):
        db = sqlite3.connect(''.join(src_gpkg))
        table = pd.read_sql_query(sql_query, db)
        table.mukey = table.mukey.astype(int)
    else:
        db = sqlite3.connect(''.join(src_gpkg[0]))
        table1 = pd.read_sql_query(sql_query, db)
        table1.mukey = table1.mukey.astype(int)

        db = sqlite3.connect(''.join(src_gpkg[1]))
        table2 = pd.read_sql_query(sql_query, db)
        table2.mukey = table2.mukey.astype(int)

        table = table1.append(table2)

    pixel_values = pd.merge(left = pixel_values, right = table,
                            how = 'left', on = 'mukey')
    pixel_values = pixel_values.iloc[:, 1].values
    pixel_values = np.reshape(pixel_values, (nrow, ncol))
    # print(pixel_values)

    # create output raster
    driver = ds.GetDriver()
    out_data = driver.Create(out_raster, ncol, nrow, 1, gdal.GDT_Float32)

    out_data.SetGeoTransform(ds.GetGeoTransform())
    out_data.SetProjection(ds.GetProjection())

    out_band = out_data.GetRasterBand(1)
    out_band.WriteArray(pixel_values)
    out_band.FlushCache()

    out_data = None
    # os.remove("temp.tif")



"""A package that enables open source workflows with the gSSURGO dataset."""
# This file allows all subdirectories in this directory to be loaded by Python
# -*- coding: utf-8 -*-
from .viz_numeric_output import viz_numeric_output
from .query_gpkg import query_gpkg
from .build_gpkg import build_gpkg
from .aoi import aoi, state_by_bbox

__all__ = ('viz_numeric_output', 'query_gpkg', 'build_gpkg', 'aoi',
           'state_by_bbox')



"""Generate a png visualization of gSSURGO output."""

import sys
import rasterio
from rasterio import plot
from matplotlib import pyplot as plt


def viz_numeric_output(input_tif, output_png):
    r"""Generate a png visualization of gSSURGO output.

    Examples
    --------
    gssurgo.viz_numeric_output("tests/nonirryield_r.tif", \
                               "tests/nonirryield_r.png")

    """
    raster = rasterio.open(input_tif)

    f, ax = plt.subplots(1, figsize=(12, 12))
    ax.imshow(raster.read(1), cmap='gray')
    plt.savefig(output_png)



"""Functions for defining an area of interest (aoi)."""

import geopandas as gpd
from matplotlib import pyplot as plt
import re
import gdal
import os
import rasterio.merge
from pyproj import Proj, transform
import pkg_resources


def state_by_bbox(fpath, ext, xmax, xmin, ymin, ymax):
    r"""Find state abbrevations from a long-lat bbox.

    Examples
    --------
    gssurgo.state_by_bbox(fpath = "tifs", ext = "tif",  xmax = -88.34945, \
                          xmin = -88.35470, ymin = 38.70095, ymax = 38.70498)

    Notes
    -----
    download:
    http://www2.census.gov/geo/tiger/GENZ2017/shp/cb_2017_us_state_500k.zip
    `ogr2ogr -f GPKG states.gpkg cb_2017_us_state_50k.shp`
    """
    states = gpd.read_file(pkg_resources.resource_filename('gssurgo',
                                                           'states.gpkg'))
    states = states[states.STATEFP.astype(int) < 60]
    states = states[~states.NAME.isin(['Hawaii', 'Alaska'])]

    states = states.cx[xmin:xmax, ymin:ymax]

    # ax = states.plot(color='white', edgecolor='black')
    # plt.show()

    keyword_base = r'.*(%s).' + ext
    keywords = re.compile(keyword_base % '|'.join(states.STUSPS))
    candidate_files = [os.path.join(fpath, f) for
                       f in os.listdir(fpath) if f.endswith('.' + ext)]

    return list(filter(keywords.match, candidate_files))


def aoi(in_raster_path, out_raster, xmin, ymax, xmax, ymin, src_tif = None):
    r"""Crop original raster to bounding box so we can read in memory.

    Examples
    --------
    gssurgo.aoi(in_raster_path = "tifs", out_raster = "path/to/aoi.tif", \
                xmax = -88.34945, xmin = -88.35470, ymin = 38.70095, \
                ymax = 38.70498)

    Notes
    -----
    https://gis.stackexchange.com/a/237412/32531

    """
    if src_tif is None:
        src_tif = state_by_bbox(fpath = in_raster_path, xmax = xmax,
                                xmin = xmin, ymin = ymin, ymax = ymax,
                                ext = "tif")

    # https://gis.stackexchange.com/a/78944/32531
    inProj = Proj(init='epsg:4326')
    outProj = Proj('+proj=aea +lat_1=29.5 +lat_2=45.5 +lat_0=23 +lon_0=-96 \
                    +x_0=0 +y_0=0 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 \
                    +units=m +no_defs')
    xmin, ymin = transform(inProj, outProj, xmin, ymin)
    xmax, ymax = transform(inProj, outProj, xmax, ymax)

    if(len(src_tif) == 1):
        src_tif = ''.join(src_tif)
        ds = gdal.Open(src_tif)
        # raster_res = ds.GetGeoTransform()[1]
        ds = gdal.Translate(out_raster, ds, projWin = [xmin, ymax, xmax, ymin])
    else:
        ds = gdal.Open(''.join(src_tif[0]))
        ds = gdal.Translate("temp1.tif", ds,
                            projWin = [xmin, ymax, xmax, ymin])

        ds = gdal.Open(''.join(src_tif[1]))
        ds = gdal.Translate("temp2.tif", ds,
                            projWin = [xmin, ymax, xmax, ymin])

        # https://gist.github.com/nishadhka/9bc758129c2949a3194b79570198f544
        d1 = rasterio.open("temp1.tif")
        d2 = rasterio.open("temp2.tif")

        dest, output_transform = rasterio.merge.merge([d1, d2])

        with d1 as src:
            out_meta = src.meta.copy()
        out_meta.update({"driver": "GTiff",
                         "height": dest.shape[1],
                         "width": dest.shape[2],
                         "transform": output_transform})
        with rasterio.open(out_raster, "w", **out_meta) as dest1:
            dest1.write(dest)



"""A package that enables open source workflows with the gSSURGO dataset."""

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gssurgo",
    version="0.0.4",
    author="Joseph Stachelek",
    author_email="stachel2@msu.edu",
    description="Python toolbox enabling an open source gSSURGO workflow",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jsta/gssurgo",
    scripts=["bin/extract_gssurgo_tif"],    
    include_package_data=True,
    packages=setuptools.find_packages(exclude=['tests']),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
